/* control - A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 */


#include <algorithm>
#include "global.h"

/*******************************************************************************
 * cStrList
 ******************************************************************************/
cStrList::cStrList() {};

void cStrList::Add(const char* s) {
  v.push_back(std::string(s));
}

void cStrList::Clear() {
  v.clear();
}

int cStrList::Count() {
  return v.size();
}

int cStrList::Find(const char* s) {
  auto it = std::find(v.begin(), v.end(), std::string(s));
  if (it == v.end())
     return -1;
  return it - v.begin();
}

const char* cStrList::operator[](int Index) {
  return v[Index].c_str();
}

void cStrList::Update(unsigned Index, const char* s) {
  if (Index < v.size()) v[Index] = s;
}



struct iso2utf8 {/*unsigned char iso;*/ unsigned char utf8[3];};

static const iso2utf8 T885915UTF8[] = {
  {/*0x80,*/ {0xC2, 0x80, 0x00}}, {/*0x81,*/ {0xC2, 0x81, 0x00}}, {/*0x82,*/ {0xC2, 0x82, 0x00}}, {/*0x83,*/ {0xC2, 0x83, 0x00}},
  {/*0x84,*/ {0xC2, 0x84, 0x00}}, {/*0x85,*/ {0xC2, 0x85, 0x00}}, {/*0x86,*/ {0xC2, 0x86, 0x00}}, {/*0x87,*/ {0xC2, 0x87, 0x00}},
  {/*0x88,*/ {0xC2, 0x88, 0x00}}, {/*0x89,*/ {0xC2, 0x89, 0x00}}, {/*0x8A,*/ {0xC2, 0x8A, 0x00}}, {/*0x8B,*/ {0xC2, 0x8B, 0x00}},
  {/*0x8C,*/ {0xC2, 0x8C, 0x00}}, {/*0x8D,*/ {0xC2, 0x8D, 0x00}}, {/*0x8E,*/ {0xC2, 0x8E, 0x00}}, {/*0x8F,*/ {0xC2, 0x8F, 0x00}},
  {/*0x90,*/ {0xC2, 0x90, 0x00}}, {/*0x91,*/ {0xC2, 0x91, 0x00}}, {/*0x92,*/ {0xC2, 0x92, 0x00}}, {/*0x93,*/ {0xC2, 0x93, 0x00}},
  {/*0x94,*/ {0xC2, 0x94, 0x00}}, {/*0x95,*/ {0xC2, 0x95, 0x00}}, {/*0x96,*/ {0xC2, 0x96, 0x00}}, {/*0x97,*/ {0xC2, 0x97, 0x00}},
  {/*0x98,*/ {0xC2, 0x98, 0x00}}, {/*0x99,*/ {0xC2, 0x99, 0x00}}, {/*0x9A,*/ {0xC2, 0x9A, 0x00}}, {/*0x9B,*/ {0xC2, 0x9B, 0x00}},
  {/*0x9C,*/ {0xC2, 0x9C, 0x00}}, {/*0x9D,*/ {0xC2, 0x9D, 0x00}}, {/*0x9E,*/ {0xC2, 0x9E, 0x00}}, {/*0x9F,*/ {0xC2, 0x9F, 0x00}},
  {/*0xA0,*/ {0xC2, 0xA0, 0x00}}, {/*0xA1,*/ {0xC2, 0xA1, 0x00}}, {/*0xA2,*/ {0xC2, 0xA2, 0x00}}, {/*0xA3,*/ {0xC2, 0xA3, 0x00}},
  {/*0xA4,*/ {0xE2, 0x82, 0xAC}}, {/*0xA5,*/ {0xC2, 0xA5, 0x00}}, {/*0xA6,*/ {0xC5, 0xA0, 0x00}}, {/*0xA7,*/ {0xC2, 0xA7, 0x00}},
  {/*0xA8,*/ {0xC5, 0xA1, 0x00}}, {/*0xA9,*/ {0xC2, 0xA9, 0x00}}, {/*0xAA,*/ {0xC2, 0xAA, 0x00}}, {/*0xAB,*/ {0xC2, 0xAB, 0x00}},
  {/*0xAC,*/ {0xC2, 0xAC, 0x00}}, {/*0xAD,*/ {0xC2, 0xAD, 0x00}}, {/*0xAE,*/ {0xC2, 0xAE, 0x00}}, {/*0xAF,*/ {0xC2, 0xAF, 0x00}},
  {/*0xB0,*/ {0xC2, 0xB0, 0x00}}, {/*0xB1,*/ {0xC2, 0xB1, 0x00}}, {/*0xB2,*/ {0xC2, 0xB2, 0x00}}, {/*0xB3,*/ {0xC2, 0xB3, 0x00}},
  {/*0xB4,*/ {0xC5, 0xBD, 0x00}}, {/*0xB5,*/ {0xC2, 0xB5, 0x00}}, {/*0xB6,*/ {0xC2, 0xB6, 0x00}}, {/*0xB7,*/ {0xC2, 0xB7, 0x00}},
  {/*0xB8,*/ {0xC5, 0xBE, 0x00}}, {/*0xB9,*/ {0xC2, 0xB9, 0x00}}, {/*0xBA,*/ {0xC2, 0xBA, 0x00}}, {/*0xBB,*/ {0xC2, 0xBB, 0x00}},
  {/*0xBC,*/ {0xC5, 0x92, 0x00}}, {/*0xBD,*/ {0xC5, 0x93, 0x00}}, {/*0xBE,*/ {0xC5, 0xB8, 0x00}}, {/*0xBF,*/ {0xC2, 0xBF, 0x00}},
  {/*0xC0,*/ {0xC3, 0x80, 0x00}}, {/*0xC1,*/ {0xC3, 0x81, 0x00}}, {/*0xC2,*/ {0xC3, 0x82, 0x00}}, {/*0xC3,*/ {0xC3, 0x83, 0x00}},
  {/*0xC4,*/ {0xC3, 0x84, 0x00}}, {/*0xC5,*/ {0xC3, 0x85, 0x00}}, {/*0xC6,*/ {0xC3, 0x86, 0x00}}, {/*0xC7,*/ {0xC3, 0x87, 0x00}},
  {/*0xC8,*/ {0xC3, 0x88, 0x00}}, {/*0xC9,*/ {0xC3, 0x89, 0x00}}, {/*0xCA,*/ {0xC3, 0x8A, 0x00}}, {/*0xCB,*/ {0xC3, 0x8B, 0x00}},
  {/*0xCC,*/ {0xC3, 0x8C, 0x00}}, {/*0xCD,*/ {0xC3, 0x8D, 0x00}}, {/*0xCE,*/ {0xC3, 0x8E, 0x00}}, {/*0xCF,*/ {0xC3, 0x8F, 0x00}},
  {/*0xD0,*/ {0xC3, 0x90, 0x00}}, {/*0xD1,*/ {0xC3, 0x91, 0x00}}, {/*0xD2,*/ {0xC3, 0x92, 0x00}}, {/*0xD3,*/ {0xC3, 0x93, 0x00}},
  {/*0xD4,*/ {0xC3, 0x94, 0x00}}, {/*0xD5,*/ {0xC3, 0x95, 0x00}}, {/*0xD6,*/ {0xC3, 0x96, 0x00}}, {/*0xD7,*/ {0xC3, 0x97, 0x00}},
  {/*0xD8,*/ {0xC3, 0x98, 0x00}}, {/*0xD9,*/ {0xC3, 0x99, 0x00}}, {/*0xDA,*/ {0xC3, 0x9A, 0x00}}, {/*0xDB,*/ {0xC3, 0x9B, 0x00}},
  {/*0xDC,*/ {0xC3, 0x9C, 0x00}}, {/*0xDD,*/ {0xC3, 0x9D, 0x00}}, {/*0xDE,*/ {0xC3, 0x9E, 0x00}}, {/*0xDF,*/ {0xC3, 0x9F, 0x00}},
  {/*0xE0,*/ {0xC3, 0xA0, 0x00}}, {/*0xE1,*/ {0xC3, 0xA1, 0x00}}, {/*0xE2,*/ {0xC3, 0xA2, 0x00}}, {/*0xE3,*/ {0xC3, 0xA3, 0x00}},
  {/*0xE4,*/ {0xC3, 0xA4, 0x00}}, {/*0xE5,*/ {0xC3, 0xA5, 0x00}}, {/*0xE6,*/ {0xC3, 0xA6, 0x00}}, {/*0xE7,*/ {0xC3, 0xA7, 0x00}},
  {/*0xE8,*/ {0xC3, 0xA8, 0x00}}, {/*0xE9,*/ {0xC3, 0xA9, 0x00}}, {/*0xEA,*/ {0xC3, 0xAA, 0x00}}, {/*0xEB,*/ {0xC3, 0xAB, 0x00}},
  {/*0xEC,*/ {0xC3, 0xAC, 0x00}}, {/*0xED,*/ {0xC3, 0xAD, 0x00}}, {/*0xEE,*/ {0xC3, 0xAE, 0x00}}, {/*0xEF,*/ {0xC3, 0xAF, 0x00}},
  {/*0xF0,*/ {0xC3, 0xB0, 0x00}}, {/*0xF1,*/ {0xC3, 0xB1, 0x00}}, {/*0xF2,*/ {0xC3, 0xB2, 0x00}}, {/*0xF3,*/ {0xC3, 0xB3, 0x00}},
  {/*0xF4,*/ {0xC3, 0xB4, 0x00}}, {/*0xF5,*/ {0xC3, 0xB5, 0x00}}, {/*0xF6,*/ {0xC3, 0xB6, 0x00}}, {/*0xF7,*/ {0xC3, 0xB7, 0x00}},
  {/*0xF8,*/ {0xC3, 0xB8, 0x00}}, {/*0xF9,*/ {0xC3, 0xB9, 0x00}}, {/*0xFA,*/ {0xC3, 0xBA, 0x00}}, {/*0xFB,*/ {0xC3, 0xBB, 0x00}},
  {/*0xFC,*/ {0xC3, 0xBC, 0x00}}, {/*0xFD,*/ {0xC3, 0xBD, 0x00}}, {/*0xFE,*/ {0xC3, 0xBE, 0x00}}, {/*0xFF,*/ {0xC3, 0xBF, 0x00}},
};

#include <cstring>

void fix_utf8_str(std::string & str) {
  std::string s = str;
  int srclen = s.size();
  unsigned char* safebuf = (unsigned char*) calloc(1 + 4 * srclen, 1);
  unsigned char* src = (unsigned char*) s.c_str();
  unsigned char* dst = safebuf;

/*------------------------------------------------------------------------------
 macro name         | code point           | n  bytes
 utf8_one_byte(s)   | U+0000 ... U+007F    | 1  0xxxxxxx
 utf8_two_byte(s)   | U+0080 ... U+07FF    | 2  110xxxxx 10xxxxxx   
 utf8_three_byte(s) | U+0800 ... U+FFFF    | 3  1110xxxx 10xxxxxx 10xxxxxx
 utf8_four_byte(s)  | U+10000 ... U+1FFFFF | 4  11110xxx 10xxxxxx 10xxxxxx 10xxxxxx
-----------------------------------------------------------------------------*/

  #define utf8_next_byte(s)  ((*s & 0xC0) == 0x80)

  #define utf8_one_byte(s)   (*s < 0x80)

  #define utf8_two_byte(s)   (((*s & 0xE0) == 0xC0) and \
                            utf8_next_byte((s+1)))

  #define utf8_three_byte(s) (((*s & 0xF0) == 0xE0) and \
                            utf8_next_byte((s+1)) and \
                            utf8_next_byte((s+2)))

  #define utf8_four_byte(s)  (((*s & 0xF8) == 0xF0) and \
                            utf8_next_byte((s+1)) and \
                            utf8_next_byte((s+2)) and \
                            utf8_next_byte((s+3)))

  while(srclen > 0) {
     if (utf8_one_byte(src)) {
        //debug_plugin("one byte 0x%.02X", *src);
        *dst++ = *src++; srclen--;
        }
     else if (srclen > 1 and utf8_two_byte(src)) {
        //debug_plugin("two byte 0x%.02X", *src);
        memcpy(dst, src, 2);
        dst+=2; src+=2; srclen-=2;
        }
     else if (srclen > 2 and utf8_three_byte(src)) {
        //debug_plugin("three byte 0x%.02X", *src);
        memcpy(dst, src, 3);
        dst+=3; src+=3; srclen-=3;
        }
     else if (srclen > 3 and utf8_four_byte(src)) {
        //debug_plugin("four byte 0x%.02X", *src);
        memcpy(dst, src, 4);
        dst+=4; src+=4; srclen-=4;
        }
     else {
        // found a non-utf8 sequence.
        // nobody knows what went wrong here, and which translation to use,
        // so assume it to be iso8859-15 and translate to utf8.
        debug_plugin("trying to fix 0x%.2X", *src);
        const unsigned char* utf8 = T885915UTF8[*src - 0x80].utf8;
        int len = 0;

        if ((*utf8 & 0xE0) == 0xC0)
           len = 2;
        else if ((*utf8 & 0xF0) == 0xE0)
           len = 3;
        else if ((*utf8 & 0xF8) == 0xF0)
           len = 4;
        
        memcpy(dst, utf8, len);
        dst+=len; src++; srclen--;
        }
     }
  str = (const char*) safebuf;
  //hexdump("str", (const unsigned char*) str.c_str(), str.size());
}





#ifdef DEBUG

/*******************************************************************************
 * hexdump
 ******************************************************************************/
#include <stdio.h>
#include <string.h>

void hexdump(const char * intro, const unsigned char * buf, int len) {
  int i, j;
  char sbuf[17];

  memset(&sbuf, 0, 17);

  printf("\t===================== %s ", intro);
  for(i = strlen(intro) + 1; i < 50; i++)
     printf("=");
  printf("\n");
  printf("\tlen = %d\n", len);
  for(i = 0; i < len; i++) {
     if ((i % 16) == 0) {
        printf("%s0x%.2X: ",i?"\n\t":"\t",(i / 16) * 16);
        }
     printf("%.2X ", *(buf + i));
     sbuf[i % 16] = *(buf + i);
     if (((i + 1) % 16) == 0) {
        // remove non-printable chars
        for(j = 0; j < 16; j++)
           if (! ((sbuf[j] > 31)  && (sbuf[j] < 127)))
              sbuf[j] = ' ';
        
        printf(": %s", sbuf);
        memset(&sbuf, 0, 17);
        }
     }
  if (len % 16) {
     for(i = 0; i < (len % 16); i++)
        if (! ((sbuf[i] > 31)  && (sbuf[i] < 127)))
           sbuf[i] = ' ';
     for(i = (len % 16); i < 16; i++)
        printf("   ");
     printf(": %s", sbuf);
     }
  printf("\n");
  printf("\t========================================================================\n");
  fflush(stdout);
}

#endif
